@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * Copyright (C) 2018-2021 - ZmartZone Holding BV - www.zmartzone.eu
+ * Copyright (C) 2018-2022 - ZmartZone Holding BV - www.zmartzone.eu
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Affero General Public License as
@@ -497,7 +497,9 @@ static bool oauth2_cache_file_set(oauth2_log_t *log, oauth2_cache_t *cache,
 	_oauth2_cache_files_clean(log, impl);
 
 	if (value == NULL) {
-		rc = _oauth2_cache_file_remove(log, path);
+		rc = (access(path, F_OK) == 0)
+			 ? _oauth2_cache_file_remove(log, path)
+			 : true;
 		goto unlock;
 	}
 
