@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * Copyright (C) 2018-2021 - ZmartZone Holding BV - www.zmartzone.eu
+ * Copyright (C) 2018-2022 - ZmartZone Holding BV - www.zmartzone.eu
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Affero General Public License as
@@ -36,6 +36,11 @@
 #include <openssl/hmac.h>
 #include <openssl/pem.h>
 
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+#include <openssl/core_names.h>
+#include <openssl/decoder.h>
+#endif
+
 #define _OAUTH2_JOSE_OPENSSL_ERR_LOG(log, function)                            \
 	oauth2_error(log, "%s failed: %s", function,                           \
 		     ERR_error_string(ERR_get_error(), NULL))
@@ -636,6 +641,7 @@ oauth2_jose_jwk_list_clone(oauth2_log_t *log, oauth2_jose_jwk_list_t *src)
 {
 	oauth2_jose_jwk_list_t *dst = NULL, *ptr = NULL, *last = NULL,
 			       *elem = NULL;
+	char *str = NULL;
 	cjose_err err;
 
 	ptr = src;
@@ -646,9 +652,17 @@ oauth2_jose_jwk_list_clone(oauth2_log_t *log, oauth2_jose_jwk_list_t *src)
 		elem->jwk->kid = oauth2_strdup(ptr->jwk->kid);
 
 		err.code = CJOSE_ERR_NONE;
-		elem->jwk->jwk = cjose_jwk_retain(ptr->jwk->jwk, &err);
+
+		// cjose_jwk_retain is not thread safe
+		str = cjose_jwk_to_json(ptr->jwk->jwk, true, &err);
+		if (str) {
+			elem->jwk->jwk =
+			    cjose_jwk_import(str, strlen(str), &err);
+			cjose_get_dealloc()(str);
+		}
+
 		if ((elem->jwk->jwk == NULL) && (err.code != CJOSE_ERR_NONE)) {
-			oauth2_error(log, "cjose_jwk_retain failed: %s",
+			oauth2_error(log, "cjose_jwk_to_json/import failed: %s",
 				     err.message);
 			oauth2_jose_jwk_list_free(log, elem);
 			continue;
@@ -917,11 +931,17 @@ static bool _oauth2_jose_jwt_verify_jwk(oauth2_log_t *log, void *rec,
 	    (strcmp(ctx->kid, kid) != 0))
 		goto end;
 
+	err.code = CJOSE_ERR_NONE;
 	if (cjose_jws_verify(ctx->jws, jwk->jwk, &err) == true) {
 		oauth2_debug(log, "cjose_jws_verify returned true");
 		ctx->verified = true;
 		// break the loop
 		rc = false;
+	} else if (err.code != CJOSE_ERR_NONE) {
+		oauth2_warn(log, "cjose_jws_verify failed: [%s:%lu %s %s]",
+			    err.file ? err.file : "<n/a>", err.line,
+			    err.function ? err.function : "<n/a>",
+			    err.message ? err.message : "");
 	}
 
 end:
@@ -1553,14 +1573,13 @@ static BIO *_oauth2_jose_str2bio(oauth2_log_t *log, const char *value)
 	BIO *input = NULL;
 
 	if ((input = BIO_new(BIO_s_mem())) == NULL) {
-		oauth2_error(log, "BIO allocation failed: ",
+		oauth2_error(log, "BIO allocation failed: %s",
 			     ERR_error_string(ERR_get_error(), NULL));
 		goto end;
 	}
 
 	if (BIO_puts(input, value) <= 0) {
-		oauth2_error(log, "BIO_puts failed: ",
-			     ERR_error_string(ERR_get_error(), NULL));
+		_OAUTH2_JOSE_OPENSSL_ERR_LOG(log, "BIO_puts");
 		goto end;
 	}
 
@@ -1578,12 +1597,15 @@ _oauth2_jose_options_jwk_set_rsa_key(oauth2_log_t *log, EVP_PKEY *pkey,
 	cjose_jwk_rsa_keyspec key_spec;
 	cjose_err err;
 	cjose_jwk_t *jwk = NULL;
-	RSA *rsa = NULL;
-	const BIGNUM *rsa_n, *rsa_e;
+	BIGNUM *rsa_n = NULL, *rsa_e = NULL;
 
 	memset(&key_spec, 0, sizeof(cjose_jwk_rsa_keyspec));
 
-	rsa = EVP_PKEY_get1_RSA(pkey);
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+	EVP_PKEY_get_bn_param(pkey, OSSL_PKEY_PARAM_RSA_N, &rsa_n);
+	EVP_PKEY_get_bn_param(pkey, OSSL_PKEY_PARAM_RSA_E, &rsa_e);
+#else
+	RSA *rsa = (RSA *)EVP_PKEY_get1_RSA(pkey);
 	if (rsa == NULL) {
 		rv = oauth2_stradd(NULL, "EVP_PKEY_get1_RSA failed", ": ",
 				   ERR_error_string(ERR_get_error(), NULL));
@@ -1591,13 +1613,15 @@ _oauth2_jose_options_jwk_set_rsa_key(oauth2_log_t *log, EVP_PKEY *pkey,
 	}
 
 #if OPENSSL_VERSION_NUMBER >= 0x10100005L && !defined(LIBRESSL_VERSION_NUMBER)
-	RSA_get0_key(rsa, &rsa_n, &rsa_e, NULL);
+	RSA_get0_key(rsa, (const BIGNUM **)&rsa_n, (const BIGNUM **)&rsa_e,
+		     NULL);
 #else
 	rsa_n = rsa->n;
 	rsa_e = rsa->e;
 #endif
 
 	RSA_free(rsa);
+#endif
 
 	key_spec.nlen = BN_num_bytes(rsa_n);
 	key_spec.n = oauth2_mem_alloc(key_spec.nlen);
@@ -1818,64 +1842,108 @@ static oauth2_jose_jwk_list_t *
 _oauth2_jose_jwks_eckey_url_resolve_response_callback(oauth2_log_t *log,
 						      char *response)
 {
-
 	oauth2_jose_jwk_list_t *keys = NULL;
-	BIO *input = NULL;
-	EC_KEY *eckey = NULL;
-	const EC_GROUP *ecgroup = NULL;
-	const EC_POINT *ecpoint = NULL;
 	BIGNUM *x = NULL, *y = NULL;
 	cjose_jwk_ec_keyspec spec;
 	cjose_jwk_t *jwk = NULL;
 	cjose_err err;
 
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+	OSSL_DECODER_CTX *dctx = NULL;
+	EVP_PKEY *pkey = NULL;
+	const unsigned char *data = (unsigned char *)response;
+	size_t datalen = strlen(response) + 1;
+
+	dctx = OSSL_DECODER_CTX_new_for_pkey(
+	    &pkey, "PEM", NULL, "EC",
+	    OSSL_KEYMGMT_SELECT_PUBLIC_KEY |
+		OSSL_KEYMGMT_SELECT_DOMAIN_PARAMETERS,
+	    NULL, NULL);
+	if (dctx == NULL) {
+		_OAUTH2_JOSE_OPENSSL_ERR_LOG(log,
+					     "OSSL_DECODER_CTX_new_for_pkey");
+		goto end;
+	}
+
+	if (OSSL_DECODER_from_data(dctx, &data, &datalen) <= 0) {
+		_OAUTH2_JOSE_OPENSSL_ERR_LOG(log, "OSSL_DECODER_from_data");
+		goto end;
+	}
+
+	char *curve_name[64];
+	size_t len = 0;
+	if (EVP_PKEY_get_utf8_string_param(pkey, OSSL_PKEY_PARAM_GROUP_NAME,
+					   (char *)curve_name,
+					   sizeof(curve_name), &len) <= 0) {
+		_OAUTH2_JOSE_OPENSSL_ERR_LOG(log,
+					     "EVP_PKEY_get_utf8_string_param");
+		goto end;
+	}
+
+	spec.crv = OBJ_txt2nid((char *)curve_name);
+	if (spec.crv == NID_undef) {
+		_OAUTH2_JOSE_OPENSSL_ERR_LOG(log, "OBJ_txt2nid");
+		goto end;
+	}
+
+	if (EVP_PKEY_get_bn_param(pkey, OSSL_PKEY_PARAM_EC_PUB_X, &x) <= 0) {
+		_OAUTH2_JOSE_OPENSSL_ERR_LOG(log, "EVP_PKEY_get_bn_param");
+		goto end;
+	}
+
+	if (EVP_PKEY_get_bn_param(pkey, OSSL_PKEY_PARAM_EC_PUB_Y, &y) <= 0) {
+		_OAUTH2_JOSE_OPENSSL_ERR_LOG(log, "EVP_PKEY_get_bn_param");
+		goto end;
+	}
+
+#else
+	BIO *input = NULL;
+	EC_KEY *eckey = NULL;
+	const EC_GROUP *ecgroup = NULL;
+	const EC_POINT *ecpoint = NULL;
+
 	input = _oauth2_jose_str2bio(log, response);
 	if (input == NULL)
 		goto end;
 
 	eckey = PEM_read_bio_EC_PUBKEY(input, NULL, 0, 0);
 	if (eckey == NULL) {
-		oauth2_error(log, "PEM_read_bio_EC_PUBKEY failed: ",
-			     ERR_error_string(ERR_get_error(), NULL));
+		_OAUTH2_JOSE_OPENSSL_ERR_LOG(log, "PEM_read_bio_EC_PUBKEY");
 		goto end;
 	}
 
 	ecgroup = EC_KEY_get0_group(eckey);
 	if (ecgroup == NULL) {
-		oauth2_error(log, "EC_KEY_get0_group failed: ",
-			     ERR_error_string(ERR_get_error(), NULL));
+		_OAUTH2_JOSE_OPENSSL_ERR_LOG(log, "EC_KEY_get0_group");
 		goto end;
 	}
 
 	spec.crv = EC_GROUP_get_curve_name(ecgroup);
 	if (spec.crv == 0) {
-		oauth2_error(log, "EC_GROUP_get_curve_name failed: ",
-			     ERR_error_string(ERR_get_error(), NULL));
+		_OAUTH2_JOSE_OPENSSL_ERR_LOG(log, "EC_GROUP_get_curve_name");
 		goto end;
 	}
 
 	ecpoint = EC_KEY_get0_public_key(eckey);
 	if (ecpoint == 0) {
-		oauth2_error(log, "EC_KEY_get0_public_key failed: ",
-			     ERR_error_string(ERR_get_error(), NULL));
+		_OAUTH2_JOSE_OPENSSL_ERR_LOG(log, "EC_KEY_get0_public_key");
 		goto end;
 	}
 
 	x = BN_new();
 	y = BN_new();
 	if ((x == NULL) || (y == NULL)) {
-		oauth2_error(log, "BN_new failed: ",
-			     ERR_error_string(ERR_get_error(), NULL));
+		_OAUTH2_JOSE_OPENSSL_ERR_LOG(log, "BN_new");
 		goto end;
 	}
 
 	if (EC_POINT_get_affine_coordinates_GFp(ecgroup, ecpoint, x, y, NULL) !=
 	    1) {
-		oauth2_error(log,
-			     "EC_POINT_get_affine_coordinates_GFp failed: ",
-			     ERR_error_string(ERR_get_error(), NULL));
+		_OAUTH2_JOSE_OPENSSL_ERR_LOG(
+		    log, "EC_POINT_get_affine_coordinates_GFp");
 		goto end;
 	}
+#endif
 
 	spec.xlen = BN_num_bytes(x);
 	spec.x = oauth2_mem_alloc(spec.xlen);
@@ -1891,8 +1959,8 @@ _oauth2_jose_jwks_eckey_url_resolve_response_callback(oauth2_log_t *log,
 	err.code = CJOSE_ERR_NONE;
 	jwk = cjose_jwk_create_EC_spec(&spec, &err);
 	if ((jwk == NULL) || (err.code != CJOSE_ERR_NONE)) {
-		oauth2_error(log,
-			     "cjose_jwk_create_EC_spec failed: ", err.message);
+		oauth2_error(log, "cjose_jwk_create_EC_spec failed: %s",
+			     err.message);
 		goto end;
 	}
 
@@ -1912,11 +1980,16 @@ end:
 	if (y)
 		BN_free(y);
 
+#if OPENSSL_VERSION_NUMBER >= 0x30000000L
+	if (dctx)
+		OSSL_DECODER_CTX_free(dctx);
+#else
 	if (eckey)
 		EC_KEY_free(eckey);
 
 	if (input)
 		BIO_free(input);
+#endif
 
 	return keys;
 }
@@ -2012,6 +2085,8 @@ char *oauth2_jose_resolve_from_uri(oauth2_log_t *log, oauth2_uri_ctx_t *uri_ctx,
 		oauth2_cache_get(log, uri_ctx->cache,
 				 oauth2_cfg_endpoint_get_url(uri_ctx->endpoint),
 				 &response);
+
+		*refresh = true;
 	}
 
 	if (response == NULL) {
