@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * Copyright (C) 2018-2021 - ZmartZone Holding BV - www.zmartzone.eu
+ * Copyright (C) 2018-2022 - ZmartZone Holding BV - www.zmartzone.eu
  *
  * This program is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Affero General Public License as
@@ -338,10 +338,11 @@ static apr_status_t oauth2_apache_request_context_free(void *rec)
 }
 
 oauth2_apache_request_ctx_t *
-oauth2_apache_request_context(request_rec *r,
+oauth2_apache_request_context(request_rec *rr,
 			      oauth2_log_function_t request_log_cb,
 			      const char *user_data_key)
 {
+	request_rec *r = (rr->main != NULL) ? rr->main : rr;
 	oauth2_apache_request_ctx_t *ctx = NULL;
 	apr_pool_userdata_get((void **)&ctx, user_data_key, r->pool);
 	if (ctx == NULL) {
